#include "GeoFun.h"

#define IMAGEFILE <GeoFun/GeoFun.iml>
#include <Draw/iml_source.h>



void DrawCanvas::DoPaint0(Painter& sw)
{
	double x1,y1;
	int cnt1,cnt2;
	
	LOG("\nTime paint0 start : " << GetTickCount());

	Size sz = GetSize();
	sw.Clear(White());
	x1 = sz.cx/2 ;
	y1 = sz.cy/2 ;
	
	cnt1 = data.GetCount();
	
	for(int i = 0; i < data.GetCount(); i++) {
		Vector<Pointf>& p = data[i].point;
		sw.Move(x1 + p[0].x, y1 + p[0].y);

		cnt2 = p.GetCount();
		
		for(int j = 1; j < p.GetCount(); j++)  
			sw.Line(x1 + p[j].x, y1 + p[j].y);

//		if(data[i].filled)  sw.Fill(data[i].color).EvenOdd(true);
		sw.Stroke(data[i].penwidth, data[i].color);
	}

	LOG("Time paint0 end  : " << GetTickCount());
	LOG("\n Path points : " << cnt2 );
}

DrawCanvas::DrawCanvas() {
	
}

void DrawCanvas::DoPaint(Painter& sw)
{
	bool painting = false ;
	if(painting) {
		PaintingPainter h(2000, 2000);
		DoPaint0(h);
		sw.Paint(h);
	}
	else

		DoPaint0(sw);
}




void DrawCanvas::Paint(Draw& drw) {
	Size sz = GetSize();
	ImageBuffer ib(sz);
	BufferPainter sw(ib);	
	
	bool transparent = false ;

	DoPaint(sw);
	LOG("\nDraw Image S : " << GetTickCount());
	drw.DrawImage(0, 0, ib);
	LOG("Draw Image E : " << GetTickCount());
}


void GeoFun::PrepareData()
{
	LOG("\n\nPrepare S : " << GetTickCount());

	int p,q,pw;

	double a, b, h ;
	double x,y;
	double Period, step, t ;
	
	step = 5.0 ; // value in deg
	
	int NumSteps = abs((int)( 2 * p * 180 / step  )) + 1 ;
		
	p = InputPane.edValP ;
	q = InputPane.edValQ ;
	pw = InputPane.edPenWidth ;
	
	Period = 2 * M_PI * p ;
	
	a = InputPane.edBaseCircleRad;
	b = (double)p * a / (double)q;
	h = InputPane.edArmLength ;
	
	InputPane.RollingRad = "Rolling Radius : " + FormatDouble(b) ;

	data.Clear();
	data.Add();
	data[0].color = InputPane.ChooseColor.GetData();
	data[0].penwidth = InputPane.edPenWidth;
	
	for( int i = 0; i<NumSteps; i++)
	{
		t = (double) i / (double) NumSteps * Period ;
		x = (a+b) * cos(t)- h * cos(((a+b)/b)*t) ;	
		y = (a+b) * sin(t)- h * sin(((a+b)/b)*t);
		Pointf p1;
		p1.x = x ;
		p1.y = y ;

		data[0].point.Add(p1);
	}
	LOG("Prepare E : " << GetTickCount());
	dc1.Refresh();
}

void GeoFun::FirstDraw()
{
	PrepareData();
}


GeoFun::GeoFun()
{
	CtrlLayout(*this, "Geometric Curves");

	CtrlLayout(InputPane);
	
	Sizeable().Zoomable();

	InputPane.btnShow << THISBACK(PrepareData);

	Add(dc1.HSizePosZ(5,160).VSizePosZ(5,5));
	Add(InputPane.RightPosZ(5,150).TopPosZ(5,350));
	BackPaint();

	data.Clear();
	
	InputPane.edArmLength.SetData(155 ) ;
	InputPane.edBaseCircleRad.SetData( 120 );
	InputPane.edPenWidth.SetData( 2 );
	InputPane.edValP.SetData( 9 );
	InputPane.edValQ.SetData( 31 );
	InputPane.ChooseColor.SetData( Blue()) ;
	
	SetTimeCallback(200, THISBACK(FirstDraw));

}

GUI_APP_MAIN
{
	GeoFun().Run();
}
